from photospicker.exception.abstract_exception import AbstractException


class UploaderException(AbstractException):
    """Exception when uploading photos"""

    # Error constants
    NOT_FOUND = 1
    NOT_EMPTY = 2



from PIL.JpegImagePlugin import JpegImageFile

from photospicker.picker.abstract_picker import AbstractPicker
from PIL import Image
from PIL import ExifTags
import operator


class LastPhotosPicker(AbstractPicker):
    """Pick the lastest photos by DateTimeOriginal in EXIF data"""

    def scan(self):
        """Scan the given path for building picked file paths list"""
        data_to_sort = {}

        scanned = 0
        self._notify_progress(scanned)
        for filepath in self._files_to_scan:
            img = Image.open(filepath)

            if isinstance(img, JpegImageFile):
                exif_data = img._getexif()

                if exif_data is None:
                    exif_data = {}

                for key in exif_data.keys():
                    if ExifTags.TAGS.get(key) != 'DateTimeOriginal':
                        continue

                    data_to_sort[filepath] = exif_data[key]

            scanned += 1
            self._notify_progress(scanned)

        self._notify_end()

        sorted_filenames = sorted(
            data_to_sort.items(),
            key=operator.itemgetter(1),
            reverse=True
        )

        for key, data in enumerate(sorted_filenames):
            if key >= self._photos_count:
                break
            self._picked_file_paths.append(data[0])



from photospicker.filter.abstract_filter import AbstractFilter  # noqa


class StartFilterEvent(object):
    """Event dispatched before a filter execution"""

    def __init__(self, photo_filter, filepath):
        """
        Constructor

        :param AbstractFilter photo_filter: filter
        :param str filepath               : file processed by the filter
        """
        self._filter_name = photo_filter.__class__.__name__
        self._filepath = filepath

    def filter_name(self):  # pragma: no cover
        """
        Getter for the filter name

        :return str
        """
        return self._filter_name

    def filepath(self):  # pragma: no cover
        """
        Getter for the file path of the processes file

        :return: str
        """
        return self._filepath



class EndUploadEvent(object):
    """Event dispatched when an upload starts"""

    def __init__(self, filepath, uploaded_files, files_to_upload):
        """
        Constructor

        :param str filepath       : file uploaded
        :param int uploaded_files : total files uploaded count
        :param int files_to_upload: total files to upload count
        """
        self._filepath = filepath
        self._uploaded_files = uploaded_files
        self._files_to_upload = files_to_upload

    @property
    def filepath(self):  # pragma: no cover
        """
        Getter for the file uploaded

        :return str
        """
        return self._filepath

    @property
    def uploaded_files(self):  # pragma: no cover
        """
        Getter for the total files uploaded count

        :return int
        """
        return self._uploaded_files

    @property
    def files_to_upload(self):  # pragma: no cover
        """
        Getter for the total files to upload count

        :return int
        """
        return self._files_to_upload



from photospicker.uploader.abstract_uploader import AbstractUploader
from dropbox.dropbox import Dropbox
from dropbox.exceptions import ApiError
from dropbox.files import DeleteError


class DropboxUploader(AbstractUploader):

    def __init__(self, api_token):
        """
        Constructor

        :param str api_token: Dropbox api token
        """
        super(DropboxUploader, self).__init__()
        self._dbx = Dropbox(api_token)

    def initialize(self):
        """Clear remote directory"""
        # Clear application directory
        try:
            self._dbx.files_delete_v2('/photos')
        except ApiError as e:
            if not isinstance(e.error, DeleteError) \
                    or not e.error.is_path_lookup():
                raise e

    def upload(self, binary, original_filename):
        """
        Upload or copy files to destination

        :param str binary           : binary data to upload
        :param str original_filename: original file name
        """
        # Upload file
        path = "/photos/{photo_name}".format(
            photo_name=self._build_filename(original_filename)
        )
        self._dbx.files_upload(binary, path)



from abc import ABCMeta, abstractmethod
import os


class AbstractUploader:
    """Abstract class for creating "upload" classes"""

    __metaclass__ = ABCMeta

    def __init__(self):
        """Constructor"""
        self._photo_counter = 0

    def initialize(self):  # pragma: no cover
        """Initialize method which can be used by subclasses"""
        pass

    @abstractmethod
    def upload(self, binary, original_filename):  # pragma: no cover
        """
        Upload or copy files to destination

        :param str binary           : binary data to upload
        :param str original_filename: original file name

        :raise NotImplementedError
        """
        raise NotImplementedError()

    def increase_photo_counter(self):
        """Increase photo counter"""
        self._photo_counter += 1

    def _build_filename(self, original_filename):
        """
        Return next photo to upload filename

        :param str original_filename: name of the original file

        :return str
        """
        filename, ext = os.path.splitext(original_filename)
        return 'photo{counter}{ext}'.format(
            counter=self._photo_counter,
            ext=ext.lower()
        )



class ScanProgressEvent(object):
    """Event dispatched during scan and holding progress data"""

    def __init__(self, files_scanned, files_to_scan, end):
        """
        Constructor

        :param int files_scanned: files scanned count
        :param int files_to_scan: total files count to scan
        :param bool end         : whether the scan is ended
        """
        self._files_scanned = files_scanned
        self._files_to_scan = files_to_scan
        self._end = end

    @property
    def files_scanned(self):  # pragma: no cover
        """
        Getter for the scanned files count

        :return int
        """
        return self._files_scanned

    @property
    def files_to_scan(self):  # pragma: no cover
        """
        Getter for the total files count to scan

        :return int
        """
        return self._files_to_scan

    @property
    def end(self):  # pragma: no cover
        """
        Getter for whether the scan is ended

        :return bool
        """
        return self._end



from abc import ABCMeta, abstractmethod
from photospicker.event.scan_progress_event import ScanProgressEvent
from zope import event
from photospicker.exception.picker_exception import PickerException
import os
import fnmatch
import string


class AbstractPicker:
    """
    Abstract class for creating "picker" classes

    A picker object select files in a path according to a strategy
    which characterizes the picker
    """

    __metaclass__ = ABCMeta

    def __init__(
            self,
            directory_paths,
            photos_count,
            patterns=None,
            excluded_patterns=None
    ):
        """
        Constructor

        :param mixed directory_paths:   directory paths to scan
        :param int   photos_count:      photos count to pick
        :param list  patterns:          patterns (in lowercase) that files must
                                        match for being scanned
        :param list  excluded_patterns: directory patterns excluded
                                        form the scan
        :raise TypeError
        """
        if isinstance(directory_paths, list):
            self._paths = directory_paths
        else:
            self._paths = [directory_paths]

        self._files_to_scan = []
        self._picked_file_paths = []
        self._photos_count = photos_count

        if patterns is None:
            patterns = ['*.tif', '*.tiff', '*.jpg', '*.jpeg', '*.png']
        elif not isinstance(patterns, list):
            raise TypeError("patterns argument must be a list")

        self._patterns = patterns

        if excluded_patterns is None:
            self._excluded_patterns = []
        else:
            self._excluded_patterns = excluded_patterns

    @property
    def picked_file_paths(self):
        """Return an array of the picked file paths"""
        return self._picked_file_paths

    def initialize(self):
        """Fill in the list of files to scan"""
        for path in self._paths:
            for root, dirnames, filenames in os.walk(path):
                if self._is_in_excluded_patterns(root):
                    continue
                for filename in filenames:
                    for pattern in self._patterns:
                        if fnmatch.fnmatch(filename.lower(), pattern):
                            self._files_to_scan.append(os.path.join(
                                root,
                                filename
                            ))
        if not self._files_to_scan:
            raise PickerException(
                PickerException.EMPTY_SCAN,
                "No photos to scan found in given directory(ies)"
            )

    def _is_in_excluded_patterns(self, path):
        """
        Check if a path match with an excluded pattern

        :param string path: path to check

        :return: bool
        """
        for excluded_pattern in self._excluded_patterns:
            if string.find(path + '/', excluded_pattern) != -1:
                return True
        return False

    @abstractmethod
    def scan(self):  # pragma: no cover
        """
        Scan the given path for building picked file paths list

        :raise NotImplementedError
        """
        raise NotImplementedError()

    def _notify_progress(self, scanned):
        """
        Notify the progress state of the scan

        :param int scanned: scanned files count
        """
        event.notify(ScanProgressEvent(
            scanned,
            len(self._files_to_scan),
            False
        ))

    def _notify_end(self):
        """Notify the end of the scan"""
        to_scan = len(self._files_to_scan)
        event.notify(ScanProgressEvent(to_scan, to_scan, True))



from abc import ABCMeta, abstractmethod
from PIL import Image  # noqa


class AbstractFilter:
    """Abstract class for creating filters"""

    __metaclass__ = ABCMeta

    @abstractmethod
    def execute(self, img, exif_data):  # pragma: no cover
        """
        Execute filter

        :param Image img     : image object to modify
        :param dict exif_data: image exif data

        :return Image
        """
        raise NotImplementedError()
